@@ -20,14 +20,15 @@ packaged into a nice, Python-friendly 'FileUpload' object.
 """
 __docformat__ = 'restructuredtext'
 
+import locale
 import re
-from types import ListType, TupleType, StringType
-from cgi import FieldStorage
 import tempfile
+from cgi import FieldStorage
 
+import six
 import zope.component
 import zope.interface
-from zope.interface import implements, directlyProvides
+from zope.interface import implementer, directlyProvides
 from zope.i18n.interfaces import IUserPreferredLanguages
 from zope.i18n.interfaces import IUserPreferredCharsets
 from zope.i18n.interfaces import IModifiableUserPreferredLanguages
@@ -42,7 +43,7 @@ from zope.publisher.interfaces.browser import IBrowserView
 from zope.publisher.interfaces.browser import IBrowserPage
 from zope.publisher.interfaces.browser import IBrowserSkinType
 from zope.publisher.interfaces.http import IHTTPRequest
-from zope.publisher.http import HTTPRequest, HTTPResponse
+from zope.publisher.http import HTTPRequest, HTTPResponse, getCharsetUsingRequest
 
 # BBB imports, this compoennts get moved from this module
 from zope.publisher.interfaces import ISkinType #BBB import
@@ -52,11 +53,13 @@ from zope.publisher.skinnable import setDefaultSkin #BBB import
 from zope.publisher.skinnable import applySkin #BBB import
 from zope.publisher.skinnable import SkinChangedEvent #BBB import
 
+from zope.publisher._compat import PYTHON2
 
-__ArrayTypes = (ListType, TupleType)
 
-start_of_header_search=re.compile('(<head[^>]*>)', re.I).search
-base_re_search=re.compile('(<base.*?>)',re.I).search
+__ArrayTypes = (list, tuple)
+
+start_of_header_search=re.compile(b'(<head[^>]*>)', re.I).search
+base_re_search=re.compile(b'(<base.*?>)',re.I).search
 isRelative = re.compile("[-_.!~*a-zA-z0-9'()@&=+$,]+(/|$)").match
 newlines = re.compile('\r\n|\n\r|\r')
 
@@ -89,7 +92,7 @@ def field2required(v):
 
 def field2int(v):
     if isinstance(v, __ArrayTypes):
-        return map(field2int, v)
+        return list(map(field2int, v))
     v = field2string(v)
     if not v:
         raise ValueError('Empty entry when <strong>integer</strong> expected')
@@ -100,7 +103,7 @@ def field2int(v):
 
 def field2float(v):
     if isinstance(v, __ArrayTypes):
-        return map(field2float, v)
+        return list(map(field2float, v))
     v = field2string(v)
     if not v:
         raise ValueError(
@@ -113,7 +116,7 @@ def field2float(v):
 
 def field2long(v):
     if isinstance(v, __ArrayTypes):
-        return map(field2long, v)
+        return list(map(field2long, v))
     v = field2string(v)
 
     # handle trailing 'L' if present.
@@ -122,7 +125,7 @@ def field2long(v):
     if not v:
         raise ValueError('Empty entry when <strong>integer</strong> expected')
     try:
-        return long(v)
+        return int(v)
     except ValueError:
         raise ValueError("A long integer was expected in the value '%s'" % v)
 
@@ -165,7 +168,7 @@ def registerTypeConverter(field_type, converter, replace=False):
     type_converters[field_type] = converter
 
 
-isCGI_NAME = {
+isCGI_NAME = lambda key: key in {
     # These fields are placed in request.environ instead of request.form.
     'SERVER_SOFTWARE' : 1,
     'SERVER_NAME' : 1,
@@ -185,12 +188,12 @@ isCGI_NAME = {
     'CONTENT_TYPE' : 1,
     'CONTENT_LENGTH' : 1,
     'SERVER_URL': 1,
-    }.has_key
+    }
 
-hide_key={
+hide_key=lambda key: key in {
     'HTTP_AUTHORIZATION':1,
     'HTTP_CGI_AUTHORIZATION': 1,
-    }.has_key
+     }
 
 class Record(object):
 
@@ -206,20 +209,20 @@ class Record(object):
         return self.__dict__[key]
 
     def __str__(self):
-        items = self.__dict__.items()
+        items = list(self.__dict__.items())
         items.sort()
         return "{" + ", ".join(["%s: %s" % item for item in items]) + "}"
 
     def __repr__(self):
-        items = self.__dict__.items()
+        items = list(self.__dict__.items())
         items.sort()
         return ("{"
             + ", ".join(["%s: %s" % (key, repr(value))
             for key, value in items]) + "}")
 
 _get_or_head = 'GET', 'HEAD'
+@implementer(IBrowserRequest, IBrowserApplicationRequest)
 class BrowserRequest(HTTPRequest):
-    implements(IBrowserRequest, IBrowserApplicationRequest)
 
     __slots__ = (
         '__provides__', # Allow request to directly provide interfaces
@@ -246,15 +249,30 @@ class BrowserRequest(HTTPRequest):
 
     def _decode(self, text):
         """Try to decode the text using one of the available charsets."""
+        # All text comes from cgi.FieldStorage.  On Python 2 it's all bytes
+        # and we must decode.  On Python 3 it's already been decoded into
+        # Unicode, using the charset we specified when instantiating the
+        # FieldStorage instance (Latin-1).
         if self.charsets is None:
             envadapter = IUserPreferredCharsets(self)
             self.charsets = envadapter.getPreferredCharsets() or ['utf-8']
+            self.charsets = [c for c in self.charsets if c != '*']
+        if not PYTHON2 and not isinstance(text, bytes):
+            if self.charsets and self.charsets[0] == 'iso-8859-1':
+                # optimization: we are trying to decode something
+                # cgi.FieldStorage already decoded for us, let's just return it
+                # rather than waste time decoding...
+                return text
+            # undo what cgi.FieldStorage did and maintain backwards compat
+            text = text.encode('latin-1')
         for charset in self.charsets:
             try:
-                text = unicode(text, charset)
+                text = text.decode(charset)
                 break
             except UnicodeError:
                 pass
+        # XXX so when none of the provided charsets works we just return bytes
+        # and let the application crash???
         return text
 
     def processInputs(self):
@@ -290,9 +308,28 @@ class BrowserRequest(HTTPRequest):
             env = env.copy()
             del env['QUERY_STRING']
 
+        if not PYTHON2 and 'QUERY_STRING' in env:
+            # According to PEP-3333, in python-3, QUERY_STRING is a string,
+            # representing 'latin-1' encoded byte array. So, if we are in python-3
+            # context, encode text as 'latin-1' first, to try to decode
+            # resulting byte array using user-supplied charset.
+            #
+            # We also need to re-encode it in locale.getpreferredencoding() so that cgi.py
+            # FieldStorage can later decode it.
+            qs = env['QUERY_STRING'].encode('latin-1')
+            env['QUERY_STRING'] = qs.decode(locale.getpreferredencoding(), 'surrogateescape')
 
+        args = {'encoding': 'latin-1'} if not PYTHON2 else {}
         fs = ZopeFieldStorage(fp=fp, environ=env,
-                              keep_blank_values=1)
+                              keep_blank_values=1, **args)
+        # On python 3.4 and up, FieldStorage explictly closes files
+        # when it is garbage collected
+        # see:
+        #   http://bugs.python.org/issue18394
+        #   https://hg.python.org/cpython/rev/c0e9ba7b26d5
+        # so we keep a reference to the FieldStorage till we are
+        # finished processing the request.
+        self.hold(fs)
 
         fslist = getattr(fs, 'list', None)
         if fslist is not None:
@@ -388,11 +425,10 @@ class BrowserRequest(HTTPRequest):
                 # skip over empty fields
                 return
 
-        # Make it unicode if not None
         if key is not None:
             key = self._decode(key)
 
-        if type(item) == StringType:
+        if isinstance(item, (str, bytes)):
             item = self._decode(item)
 
         if flags:
@@ -515,7 +551,7 @@ class BrowserRequest(HTTPRequest):
         """Insert defaults into form dictionary."""
         form = self.form
 
-        for keys, values in self.__defaults.iteritems():
+        for keys, values in self.__defaults.items():
             if not keys in form:
                 form[keys] = values
             else:
@@ -582,7 +618,7 @@ class BrowserRequest(HTTPRequest):
         d.update(self._environ)
         d.update(self._cookies)
         d.update(self.form)
-        return d.keys()
+        return list(d.keys())
 
 
     def get(self, key, default=None):
@@ -597,7 +633,11 @@ class BrowserRequest(HTTPRequest):
 class ZopeFieldStorage(FieldStorage):
 
     def make_file(self, binary=None):
-        return tempfile.NamedTemporaryFile('w+b')
+        if PYTHON2 or self._binary_file:
+            return tempfile.NamedTemporaryFile("w+b")
+        else:
+            return tempfile.NamedTemporaryFile("w+",
+                    encoding=self.encoding, newline='\n')
 
 
 class FileUpload(object):
@@ -621,7 +661,7 @@ class FileUpload(object):
             methods = ['close', 'fileno', 'flush', 'isatty',
                 'read', 'readline', 'readlines', 'seek',
                 'tell', 'truncate', 'write', 'writelines',
-                'name']
+                'name', 'seekable']
 
         d = self.__dict__
         for m in methods:
@@ -629,7 +669,9 @@ class FileUpload(object):
                 d[m] = getattr(file,m)
 
         self.headers = aFieldStorage.headers
-        filename = unicode(aFieldStorage.filename, 'UTF-8')
+        filename = aFieldStorage.filename
+        if isinstance(aFieldStorage.filename, bytes):
+            filename = aFieldStorage.filename.decode('UTF-8')
         # fix for IE full paths
         filename = filename[filename.rfind('\\')+1:].strip()
         self.filename = filename
@@ -660,8 +702,8 @@ class TestRequest(BrowserRequest):
         if kw:
             _testEnv.update(kw)
         if body_instream is None:
-            from StringIO import StringIO
-            body_instream = StringIO('')
+            from io import BytesIO
+            body_instream = BytesIO()
 
         super(TestRequest, self).__init__(body_instream, _testEnv)
         if form:
@@ -693,7 +735,7 @@ class BrowserResponse(HTTPResponse):
 
     def _implicitResult(self, body):
         content_type = self.getHeader('content-type')
-        if content_type is None:
+        if content_type is None and self._status != 304:
             if isHTML(body):
                 content_type = 'text/html'
             else:
@@ -727,9 +769,17 @@ class BrowserResponse(HTTPResponse):
                     ibase = base_re_search(body)
                     if ibase is None:
                         # Make sure the base URL is not a unicode string.
-                        base = str(self.getBase())
-                        body = ('%s\n<base href="%s" />\n%s' %
-                                (body[:index], base, body[index:]))
+                        base = self.getBase()
+                        if not isinstance(base, bytes):
+                            encoding = getCharsetUsingRequest(self._request) or 'utf-8'
+                            base = self.getBase().encode(encoding)
+                        #body = (b'%s\n<base href="%s" />\n%s' %
+                        #        (body[:index], base, body[index:]))
+                        body = b''.join([body[:index],
+                                         b'\n<base href="',
+                                         base,
+                                         b'" />\n',
+                                         body[index:]])
         return body
 
     def getBase(self):
@@ -764,6 +814,11 @@ class BrowserResponse(HTTPResponse):
 
 def isHTML(str):
      """Try to determine whether str is HTML or not."""
+     if isinstance(str, six.binary_type):
+         try:
+             str = str.decode()
+         except UnicodeDecodeError:
+             return False
      s = str.lstrip().lower()
      if s.startswith('<!doctype html'):
          return True
@@ -781,9 +836,9 @@ def normalize_lang(lang):
     lang = lang.replace(' ', '')
     return lang
 
+@zope.component.adapter(IHTTPRequest)
+@implementer(IUserPreferredLanguages)
 class BrowserLanguages(object):
-    zope.component.adapts(IHTTPRequest)
-    implements(IUserPreferredLanguages)
 
     def __init__(self, request):
         self.request = request
@@ -840,9 +895,8 @@ class NotCompatibleAdapterError(Exception):
 
 BROWSER_LANGUAGES_KEY = "zope.publisher.browser.IUserPreferredLanguages"
 
-class CacheableBrowserLanguages(BrowserLanguages):
 
-    implements(IUserPreferredLanguages)
+class CacheableBrowserLanguages(BrowserLanguages):
 
     def getPreferredLanguages(self):
         languages_data = self._getLanguagesData()
@@ -860,10 +914,9 @@ class CacheableBrowserLanguages(BrowserLanguages):
             annotations[BROWSER_LANGUAGES_KEY] = languages_data = {}
         return languages_data
 
+@implementer(IModifiableUserPreferredLanguages)
 class ModifiableBrowserLanguages(CacheableBrowserLanguages):
 
-    implements(IModifiableUserPreferredLanguages)
-
     def setPreferredLanguages(self, languages):
         languages_data = self.request.annotations.get(BROWSER_LANGUAGES_KEY)
         if languages_data is None:
@@ -876,6 +929,7 @@ class ModifiableBrowserLanguages(CacheableBrowserLanguages):
         languages_data["overridden"] = languages
         self.request.setupLocale()
 
+@implementer(IBrowserView)
 class BrowserView(Location):
     """Browser View.
 
@@ -891,7 +945,6 @@ class BrowserView(Location):
     >>> view.__parent__
     'parent'
     """
-    implements(IBrowserView)
 
     def __init__(self, context, request):
         self.context = context
@@ -905,6 +958,7 @@ class BrowserView(Location):
 
     __parent__ = property(__getParent, __setParent)
 
+@implementer(IBrowserPage)
 class BrowserPage(BrowserView):
     """Browser page
 
@@ -956,7 +1010,6 @@ class BrowserPage(BrowserView):
     It is the subclass' responsibility to do that.
 
     """
-    implements(IBrowserPage)
 
     def browserDefault(self, request):
         return self, ()
