@@ -12,10 +12,8 @@
 #
 ##############################################################################
 """Default view name API
-
-$Id$
 """
-from zope.component.interfaces import ComponentLookupError
+from zope.interface.interfaces import ComponentLookupError
 from zope.component import getSiteManager
 
 import zope.interface
@@ -61,8 +59,9 @@ def queryDefaultViewName(object, request, default=None, context=None):
       >>> import zope.interface
       >>> class IMyObject(zope.interface.Interface):
       ...   pass
-      >>> class MyObject(object):
-      ...   zope.interface.implements(IMyObject)
+      >>> @zope.interface.implementer(IMyObject)
+      ... class MyObject(object):
+      ...   pass
       >>> queryDefaultViewName(MyObject(), object()) is None
       True
 
