@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001-2002 Zope Foundation and Contributors.
+# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,12 +12,9 @@
 #
 ##############################################################################
 """FTP Publisher
-
-$Id: ftp.py 105429 2009-11-02 07:47:51Z ctheune $
 """
-from zope.authentication.loginpassword import LoginPassword
-from zope.component import adapts
-from zope.interface import implements
+import six
+from zope.interface import implementer
 from zope.publisher.interfaces.ftp import IFTPCredentials, IFTPRequest
 from zope.publisher.base import BaseResponse, BaseRequest
 
@@ -31,15 +28,15 @@ class FTPResponse(BaseResponse):
 
     def getResult(self):
         if getattr(self, '_exc', None) is not None:
-            raise self._exc[0], self._exc[1], self._exc[2]
+            six.reraise(self._exc[0], self._exc[1], self._exc[2])
         return self._result
 
     def handleException(self, exc_info):
         self._exc = exc_info
 
 
+@implementer(IFTPCredentials, IFTPRequest)
 class FTPRequest(BaseRequest):
-    implements(IFTPCredentials, IFTPRequest)
 
     __slots__ = '_auth'
 
@@ -71,23 +68,8 @@ class FTPRequest(BaseRequest):
         'See IFTPCredentials'
 
 
-class FTPAuth(LoginPassword):
-    """ILoginPassword adapter for handling common FTP authentication."""
-
-    # This was moved from zope.app.security as a part of refactoring process,
-    # see http://mail.zope.org/pipermail/zope-dev/2009-March/035325.html for
-    # the reasoning.
-
-    adapts(IFTPCredentials)
-
-    def __init__(self, request):
-        self.__request = request
-        lpw = request._authUserPW()
-        if lpw is None:
-            login, password = None, None
-        else:
-            login, password = lpw
-        super(FTPAuth, self).__init__(login, password)
-
-    def needLogin(self, realm):
-        self.__request.unauthorized("Did not work")
+# BBB
+try:
+    from zope.login.ftp import FTPAuth
+except ImportError:
+    pass
