@@ -13,7 +13,9 @@
 ##############################################################################
 """HTTP Publisher
 """
-from cStringIO import StringIO
+import sys
+import base64
+from io import BytesIO
 from zope.i18n.interfaces import IUserPreferredCharsets
 from zope.i18n.interfaces import IUserPreferredLanguages
 from zope.i18n.locales import locales, LoadLocaleError
@@ -31,25 +33,35 @@ from zope.publisher.interfaces.http import IHTTPVirtualHostChangedEvent
 from zope.publisher.interfaces.http import IResult
 from zope.publisher.interfaces.logginginfo import ILoggingInfo
 from zope.publisher.skinnable import setDefaultSkin
-import Cookie
-import cgi
 import logging
 import tempfile
 import types
-import urllib
-import urlparse
 import zope.component
 import zope.contenttype.parse
 import zope.event
 import zope.interface
 
+from zope.publisher._compat import PYTHON2, CLASS_TYPES, to_unicode
+
+if PYTHON2:
+    import Cookie as cookies
+    from urllib import quote
+    from urlparse import urlsplit
+    from cgi import escape
+else:
+    import http.cookies as cookies
+    from urllib.parse import quote, urlsplit
+    from html import escape
+    unicode = str
+    basestring = (str, bytes)
 
 # Default Encoding
 ENCODING = 'UTF-8'
 
 # not just text/* but RFC 3023 and */*+xml
 import re
-unicode_mimetypes_re = re.compile(r"^text\/.*$|^.*\/xml.*$|^.*\+xml$")
+unicode_mimetypes_re = re.compile(
+    r"^text\/.*$|^.*\/xml.*$|^.*\+xml$|^application/json$")
 
 eventlog = logging.getLogger('eventlog')
 
@@ -59,6 +71,37 @@ class CookieMapper(RequestDataMapper):
 class HeaderGetter(RequestDataGetter):
     _gettrname = 'getHeader'
 
+
+host_port_re = re.compile(
+    r"^(.*):([0-9]*)$", re.DOTALL)
+
+def splitport(host):
+    """Split port number off the hostname.
+
+        >>> splitport('example.com:80')
+        ('example.com', '80')
+
+        >>> splitport('localhost')
+        ('localhost', None)
+
+        >>> splitport('[::1]')
+        ('[::1]', None)
+
+        >>> splitport('[::1]:443')
+        ('[::1]', '443')
+
+        >>> splitport('localhost:')
+        ('localhost', None)
+
+    """
+    match = host_port_re.match(host)
+    if match:
+        host, port = match.groups()
+    else:
+        port = None
+    return host, port or None
+
+
 def sane_environment(env):
     # return an environment mapping which has been cleaned of
     # funny business such as REDIRECT_ prefixes added by Apache
@@ -72,11 +115,14 @@ def sane_environment(env):
     if 'HTTP_CGI_AUTHORIZATION' in dict:
         dict['HTTP_AUTHORIZATION'] = dict.pop('HTTP_CGI_AUTHORIZATION')
     if 'PATH_INFO' in dict:
-        dict['PATH_INFO'] = dict['PATH_INFO'].decode('utf-8')
+        # Recode PATH_INFO to UTF-8 from original latin1
+        pi = dict['PATH_INFO']
+        pi = pi if isinstance(pi, bytes) else pi.encode('latin1')
+        dict['PATH_INFO'] = pi.decode(ENCODING)
     return dict
 
+@zope.interface.implementer(IHTTPVirtualHostChangedEvent)
 class HTTPVirtualHostChangedEvent(object):
-    zope.interface.implements(IHTTPVirtualHostChangedEvent)
 
     request = None
 
@@ -176,8 +222,8 @@ class URLGetter(object):
                 return self.__request.getURL(i)
             else:
                 return self.__request.getApplicationURL(i)
-        except IndexError, v:
-            if v[0] == i:
+        except IndexError as v:
+            if v.args[0] == i:
                 return default
             raise
 
@@ -195,7 +241,7 @@ class HTTPInputStream(object):
         if not size:
             size = environment.get('HTTP_CONTENT_LENGTH')
         if not size or int(size) < 65536:
-            self.cacheStream = StringIO()
+            self.cacheStream = BytesIO()
         else:
             self.cacheStream = tempfile.TemporaryFile()
         self.size = size and int(size) or -1
@@ -225,12 +271,15 @@ class HTTPInputStream(object):
 
     def readlines(self, hint=0):
         data = self.stream.readlines(hint)
-        self.cacheStream.write(''.join(data))
+        self.cacheStream.write(b''.join(data))
         return data
 
 
 DEFAULT_PORTS = {'http': '80', 'https': '443'}
 
+@zope.interface.implementer(IHTTPCredentials,
+                            IHTTPRequest,
+                            IHTTPApplicationRequest)
 class HTTPRequest(BaseRequest):
     """Model HTTP request data.
 
@@ -276,9 +325,6 @@ class HTTPRequest(BaseRequest):
     values will be looked up in the order: environment variables,
     other variables, form data, and then cookies.
     """
-    zope.interface.implements(IHTTPCredentials,
-                              IHTTPRequest,
-                              IHTTPApplicationRequest)
 
     __slots__ = (
         '__provides__',   # Allow request to directly provide interfaces
@@ -352,7 +398,7 @@ class HTTPRequest(BaseRequest):
         script = get_env('SCRIPT_NAME', '').strip()
 
         # _script and the other _names are meant for URL construction
-        self._app_names = filter(None, script.split('/'))
+        self._app_names = [f for f in script.split('/') if f]
 
         # get server URL and store it too, since we are already looking it up
         server_url = get_env('SERVER_URL', None)
@@ -379,9 +425,9 @@ class HTTPRequest(BaseRequest):
         else:
             protocol = 'http'
 
-        if environ.has_key('HTTP_HOST'):
+        if 'HTTP_HOST' in environ:
             host = environ['HTTP_HOST'].strip()
-            hostname, port = urllib.splitport(host)
+            hostname, port = splitport(host)
         else:
             hostname = environ.get('SERVER_NAME', '').strip()
             port = environ.get('SERVER_PORT', '')
@@ -401,13 +447,21 @@ class HTTPRequest(BaseRequest):
 
         # ignore cookies on a CookieError
         try:
-            c = Cookie.SimpleCookie(text)
-        except Cookie.CookieError, e:
-            eventlog.warn(e)
+            c = cookies.SimpleCookie(text)
+        except cookies.CookieError as e:
+            eventlog.warning(e)
             return result
 
-        for k,v in c.items():
-            result[unicode(k, ENCODING)] = unicode(v.value, ENCODING)
+        for k, v in c.items():
+            # recode cookie value to ENCODING (UTF-8)
+            if not isinstance(k, bytes):
+                k = k.encode('latin1')
+            rk = k.decode(ENCODING)
+            v = v.value
+            if not isinstance(v, bytes):
+                v = v.encode('latin1')
+            rv = v.decode(ENCODING)
+            result[rk] = rv
 
         return result
 
@@ -491,7 +545,9 @@ class HTTPRequest(BaseRequest):
         'See IHTTPCredentials'
         if self._auth and self._auth.lower().startswith('basic '):
             encoded = self._auth.split(None, 1)[-1]
-            name, password = encoded.decode("base64").split(':', 1)
+            decoded = base64.b64decode(encoded.encode('iso-8859-1'))
+            name, password = bytes.split(decoded, b':', 1)
+            #name, password = base64.b64decode(encoded.encode('ascii')).split(':', 1)
             return name, password
 
     def unauthorized(self, challenge):
@@ -521,8 +577,7 @@ class HTTPRequest(BaseRequest):
                 raise IndexError(level)
             names = names[:-level]
         # See: http://www.ietf.org/rfc/rfc2718.txt, Section 2.2.5
-        names = [urllib.quote(name.encode("utf-8"), safe='/+@')
-                 for name in names]
+        names = [quote(name.encode("utf-8"), safe='/+@') for name in names]
 
         if path_only:
             if not names:
@@ -544,8 +599,7 @@ class HTTPRequest(BaseRequest):
             names = self._app_names
 
         # See: http://www.ietf.org/rfc/rfc2718.txt, Section 2.2.5
-        names = [urllib.quote(name.encode("utf-8"), safe='/+@')
-                 for name in names]
+        names = [quote(name.encode("utf-8"), safe='/+@') for name in names]
 
         if path_only:
             return names and ('/' + '/'.join(names)) or '/'
@@ -608,8 +662,8 @@ class HTTPRequest(BaseRequest):
 
 
 
+@zope.interface.implementer(IHTTPResponse, IHTTPApplicationResponse)
 class HTTPResponse(BaseResponse):
-    zope.interface.implements(IHTTPResponse, IHTTPApplicationResponse)
 
     __slots__ = (
         'authUser',             # Authenticated user string
@@ -701,7 +755,7 @@ class HTTPResponse(BaseResponse):
         result.append(
             ("X-Powered-By", "Zope (www.zope.org), Python (www.python.org)"))
 
-        for key, values in headers.items():
+        for key, values in sorted(headers.items(), key=lambda x: x[0].lower()):
             if key.lower() == key:
                 # only change non-literal header names
                 key = '-'.join([k.capitalize() for k in key.split('-')])
@@ -767,7 +821,9 @@ class HTTPResponse(BaseResponse):
                 if isinstance(result, basestring):
                     r = result
                 elif result is None:
-                    r = ''
+                    # Default to bytes because unicode results require a
+                    # corresponding Content-Type header.
+                    r = b''
                 else:
                     raise TypeError(
                         'The result should be None, a string, or adaptable to '
@@ -784,7 +840,7 @@ class HTTPResponse(BaseResponse):
 
     def consumeBody(self):
         'See IHTTPResponse'
-        return ''.join(self._result)
+        return b''.join(self._result)
 
 
     def consumeBodyIter(self):
@@ -794,31 +850,37 @@ class HTTPResponse(BaseResponse):
 
     def _implicitResult(self, body):
         encoding = getCharsetUsingRequest(self._request) or 'utf-8'
-        content_type = self.getHeader('content-type')
+        content_type = self.getHeader('content-type') or ''
 
         if isinstance(body, unicode):
-            if not unicode_mimetypes_re.match(content_type):
+            ct = content_type
+            if not unicode_mimetypes_re.match(ct):
                 raise ValueError(
-                    'Unicode results must have a text, RFC 3023, or '
-                    '+xml content type.')
+                    'Unicode results must have a text, RFC 3023, RFC 4627,'
+                    ' or +xml content type.')
 
-            major, minor, params = zope.contenttype.parse.parse(content_type)
+            major, minor, params = zope.contenttype.parse.parse(ct)
 
             if 'charset' in params:
                 encoding = params['charset']
 
             try:
                 body = body.encode(encoding)
-            except UnicodeEncodeError:
+            except (UnicodeEncodeError, LookupError):
                 # RFC 2616 section 10.4.7 allows us to return an
                 # unacceptable encoding instead of 406 Not Acceptable
                 # response.
                 encoding = 'utf-8'
                 body = body.encode(encoding)
 
-            params['charset'] = encoding
-            content_type = "%s/%s;" % (major, minor)
-            content_type += ";".join(k + "=" + v for k, v in params.items())
+            if (major, minor) != ('application', 'json'):
+                # The RFC says this is UTF-8, and the type has no params.
+                params['charset'] = encoding
+            content_type = "%s/%s" % (major, minor)
+            if params:
+                content_type += ";"
+                content_type += ";".join(k + "=" + v
+                                         for k, v in params.items())
 
         if content_type:
             headers = [('content-type', content_type),
@@ -833,13 +895,13 @@ class HTTPResponse(BaseResponse):
         Calls self.setBody() with an error response.
         """
         t, v = exc_info[:2]
-        if isinstance(t, (types.ClassType, type)):
+        if isinstance(t, CLASS_TYPES):
             if issubclass(t, Redirect):
-                self.redirect(v.getLocation())
+                self.redirect(v.getLocation(), trusted=v.getTrusted())
                 return
             title = tname = t.__name__
         else:
-            title = tname = unicode(t)
+            title = tname = to_unicode(t)
 
         # Throwing non-protocol-specific exceptions is a good way
         # for apps to control the status code.
@@ -854,14 +916,13 @@ class HTTPResponse(BaseResponse):
         self.setStatus(500, u"The engines can't take any more, Jim!")
 
     def _html(self, title, content):
-        t = cgi.escape(title)
+        t = escape(title)
         return (
             u"<html><head><title>%s</title></head>\n"
             u"<body><h2>%s</h2>\n"
             u"%s\n"
-            u"</body></html>\n" %
-            (t, t, content)
-            )
+            u"</body></html>\n"
+        ) % (t, t, content)
 
     def retry(self):
         """
@@ -876,6 +937,8 @@ class HTTPResponse(BaseResponse):
         # convertable to string, for example, an URLGetter instance
         location = str(location)
 
+        __traceback_info__ = location
+
         if not trusted:
             target_host = extract_host(location)
             if target_host:
@@ -900,13 +963,22 @@ class HTTPResponse(BaseResponse):
 
     def _cookie_list(self):
         try:
-            c = Cookie.SimpleCookie()
-        except Cookie.CookieError, e:
-            eventlog.warn(e)
+            c = cookies.SimpleCookie()
+        except cookies.CookieError as e:
+            eventlog.warning(e)
             return []
         for name, attrs in self._cookies.items():
             name = str(name)
-            c[name] = attrs['value'].encode(ENCODING)
+
+            # In python-2.x, Cookie module expects plain bytes (not unicode).
+            # However, in python-3.x, latin-1 unicode string is expected (not
+            # bytes).  We make this distinction clear here.
+            cookieval = attrs['value'].encode(ENCODING)
+            if PYTHON2:
+                c[name] = cookieval
+            else:
+                c[name] = cookieval.decode('latin-1')
+
             for k,v in attrs.items():
                 if k == 'value':
                     continue
@@ -918,7 +990,7 @@ class HTTPResponse(BaseResponse):
                     k = 'max-age'
                 elif k == 'comment':
                     # Encode rather than throw an exception
-                    v = urllib.quote(v.encode('utf-8'), safe="/?:@&+")
+                    v = quote(v.encode('utf-8'), safe="/?:@&+")
                 c[name][k] = str(v)
         return str(c).splitlines()
 
@@ -929,16 +1001,15 @@ class HTTPResponse(BaseResponse):
             "for more information."
             )
 
-def sort_charsets(x, y):
-    if y[1] == 'utf-8':
-        return 1
-    if x[1] == 'utf-8':
-        return -1
-    return cmp(y, x)
-
+def sort_charsets(charset):
+    # Make utf-8 to be the last element of the sorted list
+    if charset[1] == 'utf-8':
+        return (1, charset)
+    # Otherwise, sort by charset
+    return (0, charset)
 
 def extract_host(url):
-    scheme, host, path, query, fragment = urlparse.urlsplit(url)
+    scheme, host, path, query, fragment = urlsplit(url)
     if ':' not in host:
         port = DEFAULT_PORTS.get(scheme)
         if port:
@@ -946,9 +1017,9 @@ def extract_host(url):
     return host
 
 
+@zope.interface.implementer(IUserPreferredCharsets)
+@zope.component.adapter(IHTTPRequest)
 class HTTPCharsets(object):
-    zope.component.adapts(IHTTPRequest)
-    zope.interface.implements(IUserPreferredCharsets)
 
     def __init__(self, request):
         self.request = request
@@ -996,10 +1067,12 @@ class HTTPCharsets(object):
         # range , unlike many other encodings. Since Zope can easily use very
         # different ranges, like providing a French-Chinese dictionary, it is
         # always good to use UTF-8.
-        charsets.sort(sort_charsets)
+        charsets.sort(key=sort_charsets, reverse=True)
         charsets = [charset for quality, charset in charsets]
         if sawstar and 'utf-8' not in charsets:
             charsets.insert(0, 'utf-8')
+        elif charsets == []:
+            charsets = ['utf-8']
         return charsets
 
 
@@ -1019,6 +1092,7 @@ def getCharsetUsingRequest(request):
     return charset
 
 
+@zope.interface.implementer(IResult)
 class DirectResult(object):
     """A generic result object.
 
@@ -1026,12 +1100,13 @@ class DirectResult(object):
     application to specify all headers related to the content, such as the
     content type and length.
     """
-    zope.interface.implements(IResult)
 
     def __init__(self, body):
         self.body = body
 
     def __iter__(self):
+        if isinstance(self.body, bytes):
+            return iter([self.body])
         return iter(self.body)
 
 
