@@ -18,7 +18,7 @@ __docformat__ = "reStructuredText"
 from zope.browser.interfaces import IView # BBB import
 from zope.interface import Interface
 from zope.interface import Attribute
-from zope.interface import implements
+from zope.interface import implementer
 from zope.interface.interfaces import IInterface
 from zope.interface.common.mapping import IEnumerableMapping
 from zope.interface.common.interfaces import IException
@@ -27,26 +27,43 @@ from zope.security.interfaces import IParticipation
 from zope.security.interfaces import Unauthorized
 
 class IPublishingException(IException):
-    pass
+    """
+    An exception that occurs during publication.
+    """
 
+@implementer(IPublishingException)
 class PublishingException(Exception):
-    implements(IPublishingException)
+    """
+    Default implementation of `IPublishingException`.
+    """
 
 class ITraversalException(IPublishingException):
-    pass
+    """
+    An exception that occurs during publication traversal.
+    """
 
+@implementer(ITraversalException)
 class TraversalException(PublishingException):
-    implements(ITraversalException)
+    """
+    Default implementation of `ITraversalException`.
+    """
 
 class INotFound(ILookupError, ITraversalException):
+    """
+    The object we want to traverse to cannot be found.
+    """
+
     def getObject():
         'Returns the object that was being traversed.'
 
     def getName():
         'Returns the name that was being traversed.'
 
+@implementer(INotFound)
 class NotFound(LookupError, TraversalException):
-    implements(INotFound)
+    """
+    Default implementation of `INotFound`.
+    """
 
     def __init__(self, ob, name, request=None):
         self.ob = ob
@@ -60,20 +77,26 @@ class NotFound(LookupError, TraversalException):
 
     def __str__(self):
         try:
-            ob = `self.ob`
+            ob = repr(self.ob)
         except:
             ob = 'unprintable object'
-        return 'Object: %s, name: %s' % (ob, `self.name`)
+        return 'Object: %s, name: %s' % (ob, repr(self.name))
 
 class IDebugError(ITraversalException):
+    """
+    A debug error.
+    """
     def getObject():
         'Returns the object being traversed.'
 
     def getMessage():
         'Returns the debug message.'
 
+@implementer(IDebugError)
 class DebugError(TraversalException):
-    implements(IDebugError)
+    """
+    Default implementation of `IDebugError`.
+    """
 
     message = None # override this not to cause warnings in python 2.6
 
@@ -91,12 +114,18 @@ class DebugError(TraversalException):
         return self.message
 
 class IBadRequest(IPublishingException):
+    """
+    The request is bad.
+    """
+
     def __str__():
         'Returns the error message.'
 
+@implementer(IBadRequest)
 class BadRequest(PublishingException):
-
-    implements(IBadRequest)
+    """
+    Default implementation of `IBadRequest`.
+    """
 
     message = None # override this not to cause warnings in python 2.6
 
@@ -107,30 +136,47 @@ class BadRequest(PublishingException):
         return self.message
 
 class IRedirect(IPublishingException):
+    """
+    An exception that redirects the client.
+    """
+
     def getLocation():
         'Returns the location.'
 
+    def getTrusted():
+        'Returns the trusted value.'
+
+@implementer(IRedirect)
 class Redirect(PublishingException):
+    """
+    Default implementation of `IRedirect`.
+    """
 
-    implements(IRedirect)
-
-    def __init__(self, location):
+    def __init__(self, location, trusted=False):
         self.location = location
+        self.trusted = trusted
 
     def getLocation(self):
         return self.location
 
+    def getTrusted(self):
+        return self.trusted
+
     def __str__(self):
         return 'Location: %s' % self.location
 
 class IRetry(IPublishingException):
+    """
+    An exception that indicates a request should be retried.
+    """
     def getOriginalException():
         'Returns the original exception object.'
 
+@implementer(IRetry)
 class Retry(PublishingException):
-    """Raise this to retry a request."""
-
-    implements(IRetry)
+    """
+    Default implementation of `IRetry`.
+    """
 
     def __init__(self, orig_exc=None):
         """orig_exc must be a 3-tuple as returned from sys.exc_info() or None"""
@@ -161,6 +207,9 @@ class IExceptionSideEffects(Interface):
 
 
 class IPublishTraverse(Interface):
+    """
+    Traversal for the specific purpose of publishing.
+    """
 
     def publishTraverse(request, name):
         """Lookup a name
@@ -178,11 +227,14 @@ class IPublishTraverse(Interface):
 
 
 class IPublisher(Interface):
+    """
+    An object that can publish.
+    """
 
     def publish(request):
         """Publish a request
 
-        The request must be an IPublisherRequest.
+        The request must be an `IPublisherRequest`.
         """
 
 class IResponse(Interface):
@@ -289,7 +341,7 @@ class IPublication(Interface):
 
 
 class IPublicationRequest(IParticipation):
-    """Interface provided by requests to IPublication objects
+    """Interface provided by requests to `IPublication` objects
     """
 
     response = Attribute("""The request's response object
@@ -479,29 +531,27 @@ class IStartRequestEvent(IRequestEvent):
 
 class RequestEvent(object):
     """Events for requests.
-    
-    :ivar request: The request the event is for.
+
+    :param request: The request the event is for.
     """
 
     def __init__(self, request):
         self.request = request
 
 
+@implementer(IEndRequestEvent)
 class EndRequestEvent(RequestEvent):
     """An event which gets sent when the publication is ended"""
 
-    implements(IEndRequestEvent)
-
     def __init__(self, ob, request):
         super(EndRequestEvent, self).__init__(request)
         self.object = ob
 
 
+@implementer(IStartRequestEvent)
 class StartRequestEvent(RequestEvent):
     """An event send when before publication of a request."""
 
-    implements(IStartRequestEvent)
-
 
 class ISkinType(IInterface):
     """Base interface for skin types."""
@@ -509,7 +559,7 @@ class ISkinType(IInterface):
 
 class ISkinnable(Interface):
     """A skinnable (request) can provide a skin.
-    
+
     The implementation in BrowserRequest will apply a default skin/layer called
     ``IDefaultBrowserLayer`` if not default skin get registered.
     """
