@@ -11,23 +11,26 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-"""An adapter from zope.security.interfaces.IPrincipal to
-zope.publisher.interfaces.logginginfo.ILoggingInfo.
+"""Adapter:
 
-$Id: principallogging.py 105429 2009-11-02 07:47:51Z ctheune $
+Adapts zope.security.interfaces.IPrincipal to
+zope.publisher.interfaces.logginginfo.ILoggingInfo.
 """
-from zope.component import adapts
-from zope.interface import implements
+from zope.component import adapter
+from zope.interface import implementer
 from zope.publisher.interfaces.logginginfo import ILoggingInfo
 from zope.security.interfaces import IPrincipal
 
-class PrincipalLogging(object):
 
-    adapts(IPrincipal)
-    implements(ILoggingInfo)
+@adapter(IPrincipal)
+@implementer(ILoggingInfo)
+class PrincipalLogging(object):
+    """Adapts zope.security.interfaces.IPrincipal to
+    zope.publisher.interfaces.logginginfo.ILoggingInfo."""
 
     def __init__(self, principal):
         self.principal = principal
 
     def getLogMessage(self):
-        return str(self.principal.id)
+        pid = self.principal.id
+        return pid.encode('ascii', 'backslashreplace')
