@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 ##############################################################################
 #
 # Copyright (c) 2001, 2002 Zope Foundation and Contributors.
@@ -14,9 +15,9 @@
 
 import sys
 import unittest
-from StringIO import StringIO
+from io import BytesIO
 
-from zope.interface import implements, directlyProvides, Interface
+from zope.interface import implementer, directlyProvides, Interface
 from zope.interface.verify import verifyObject
 
 from zope.publisher.publish import publish as publish_
@@ -29,6 +30,7 @@ from zope.publisher.interfaces.browser import IBrowserRequest
 from zope.publisher.interfaces.browser import IBrowserPublication
 from zope.publisher.base import DefaultPublication
 
+from zope.publisher._compat import PYTHON2
 from zope.publisher.tests.test_http import HTTPTests
 from zope.publisher.tests.publication import TestPublication
 
@@ -39,15 +41,30 @@ from zope.publisher.tests.basetestipublisherrequest \
 from zope.publisher.tests.basetestiapplicationrequest \
      import BaseTestIApplicationRequest
 
-LARGE_FILE_BODY = """-----------------------------1
+
+EMPTY_FILE_BODY = b"""-----------------------------1
+Content-Disposition: form-data; name="upload"; filename=""
+Content-Type: application/octet-stream
+
+-----------------------------1--
+"""
+
+LARGE_FILE_BODY = b''.join([b"""-----------------------------1
 Content-Disposition: form-data; name="upload"; filename="test"
 Content-Type: text/plain
 
-Here comes some text! %s
+Here comes some text! """, (b'test'*1000), b"""
 -----------------------------1--
-""" % ('test' * 1000)
+"""])
 
-IE_FILE_BODY = """-----------------------------1
+LARGE_POSTED_VALUE = b''.join([b"""-----------------------------1
+Content-Disposition: form-data; name="upload"
+
+Here comes some text! """, (b'test'*1000), b"""
+-----------------------------1--
+"""])
+
+IE_FILE_BODY = b"""-----------------------------1
 Content-Disposition: form-data; name="upload"; filename="C:\\Windows\\notepad.exe"
 Content-Type: text/plain
 
@@ -55,6 +72,9 @@ Some data
 -----------------------------1--
 """
 
+if not PYTHON2:
+    unicode = str
+
 
 def publish(request):
     publish_(request, handle_errors=0)
@@ -101,7 +121,7 @@ class BrowserTests(HTTPTests):
         class Item(object):
             """Required docstring for the publisher."""
             def __call__(self, a, b):
-                return u"%s, %s" % (`a`, `b`)
+                return u"%s, %s" % (repr(a).lstrip('u'), repr(b).lstrip('u'))
 
         class Item3(object):
             """Required docstring for the publisher."""
@@ -115,7 +135,7 @@ class BrowserTests(HTTPTests):
 
             def index(self, a, b):
                 """Required docstring for the publisher."""
-                return u"%s, %s" % (`a`, `b`)
+                return u"%s, %s" % (repr(a).lstrip('u'), repr(b).lstrip('u'))
 
         class Item2(object):
             """Required docstring for the publisher."""
@@ -131,35 +151,35 @@ class BrowserTests(HTTPTests):
         self.app.folder.item2 = Item2()
         self.app.folder.item3 = Item3()
 
-    def _createRequest(self, extra_env={}, body=""):
+    def _createRequest(self, extra_env={}, body=b""):
         env = self._testEnv.copy()
         env.update(extra_env)
         if len(body):
             env['CONTENT_LENGTH'] = str(len(body))
 
         publication = Publication(self.app)
-        instream = StringIO(body)
+        instream = BytesIO(body)
         request = TestBrowserRequest(instream, env)
         request.setPublication(publication)
         return request
 
     def testTraversalToItem(self):
         res = self._publisherResults()
-        self.failUnlessEqual(
+        self.assertEqual(
             res,
             "Status: 200 Ok\r\n"
-            "Content-Length: 7\r\n"
+            "X-Powered-By: Zope (www.zope.org), Python (www.python.org)\r\n"
+            "Content-Length: 6\r\n"
             "Content-Type: text/plain;charset=utf-8\r\n"
             "X-Content-Type-Warning: guessed from content\r\n"
-            "X-Powered-By: Zope (www.zope.org), Python (www.python.org)\r\n"
             "\r\n"
-            "u'5', 6")
+            "'5', 6")
 
     def testNoDefault(self):
         request = self._createRequest()
         response = request.response
         publish(request)
-        self.failIf(response.getBase())
+        self.assertFalse(response.getBase())
 
     def testDefault(self):
         extra = {'PATH_INFO': '/folder/item2'}
@@ -171,7 +191,7 @@ class BrowserTests(HTTPTests):
 
     def testDefaultPOST(self):
         extra = {'PATH_INFO': '/folder/item2', "REQUEST_METHOD": "POST"}
-        request = self._createRequest(extra, body='a=5&b:int=6')
+        request = self._createRequest(extra, body=b'a=5&b:int=6')
         response = request.response
         publish(request)
         self.assertEqual(response.getBase(),
@@ -182,12 +202,12 @@ class BrowserTests(HTTPTests):
         """Produce a Fieldstorage with a name wich is None, this
         should be catched"""
 
-        extra = {'REQUEST_METHOD':'POST',
-                 'PATH_INFO': u'/',
+        extra = {'REQUEST_METHOD': 'POST',
+                 'PATH_INFO': u"/",
                  'CONTENT_TYPE': 'multipart/form-data;\
                  boundary=---------------------------1'}
 
-        body = """-----------------------------1
+        body = b"""-----------------------------1
         Content-Disposition: form-data; name="field.contentType"
         ...
         application/octet-stream
@@ -200,19 +220,43 @@ class BrowserTests(HTTPTests):
         """Produce a Fieldstorage with a file handle that exposes
         its filename."""
 
-        extra = {'REQUEST_METHOD':'POST',
-                 'PATH_INFO': u'/',
+        extra = {'REQUEST_METHOD': 'POST',
+                 'PATH_INFO': u"/",
                  'CONTENT_TYPE': 'multipart/form-data;\
                  boundary=---------------------------1'}
 
         request  = self._createRequest(extra, body=LARGE_FILE_BODY)
         request.processInputs()
-        self.assert_(request.form['upload'].name)
+        self.assertTrue(request.form['upload'].name)
 
         request  = self._createRequest(extra, body=IE_FILE_BODY)
         request.processInputs()
-        self.assertEquals(request.form['upload'].filename, 'notepad.exe')
+        self.assertEqual(request.form['upload'].filename, 'notepad.exe')
 
+        # Test that we can actually read the file data
+        self.assertEqual(request.form['upload'].read(), b'Some data')
+
+        if not PYTHON2:
+            # File objects on Python 3 have a seekable() method
+            self.assertTrue(request.form['upload'].seekable())
+
+    def testEmptyFilePost(self):
+        extra = {'REQUEST_METHOD':'POST',
+                 'PATH_INFO': u"/",
+                 'CONTENT_TYPE': 'multipart/form-data;\
+                 boundary=---------------------------1'}
+
+        request  = self._createRequest(extra, body=EMPTY_FILE_BODY)
+        request.processInputs()
+
+    def testLargePostValue(self):
+        extra = {'REQUEST_METHOD':'POST',
+                 'PATH_INFO': u"/",
+                 'CONTENT_TYPE': 'multipart/form-data;\
+                 boundary=---------------------------1'}
+
+        request  = self._createRequest(extra, body=LARGE_POSTED_VALUE)
+        request.processInputs()
 
     def testDefault2(self):
         extra = {'PATH_INFO': '/folder/item2/view'}
@@ -227,14 +271,14 @@ class BrowserTests(HTTPTests):
         request = self._createRequest(extra)
         response = request.response
         publish(request)
-        self.failIf(response.getBase())
+        self.assertFalse(response.getBase())
 
     def testDefault4(self):
         extra = {'PATH_INFO': '/folder/item2/view/'}
         request = self._createRequest(extra)
         response = request.response
         publish(request)
-        self.failIf(response.getBase())
+        self.assertFalse(response.getBase())
 
     def testDefault6(self):
         extra = {'PATH_INFO': '/folder/item2/'}
@@ -258,10 +302,80 @@ class BrowserTests(HTTPTests):
         request = self._createRequest()
         publish(request)
         self.assertEqual(request.form,
-                         {u'a':u'5', u'b':6})
+                         {u"a": u"5", u"b": 6})
+
+    def testFormMultipartUTF8(self):
+        extra = {
+            'REQUEST_METHOD': 'POST',
+            'CONTENT_TYPE': 'multipart/form_data; boundary=-123',
+        }
+        body = b'\n'.join([
+            b'---123',
+            b'Content-Disposition: form-data; name="a"',
+            b'',
+            b'5',
+            b'---123',
+            b'Content-Disposition: form-data; name="b:int"',
+            b'',
+            b'6',
+            b'---123',
+            b'Content-Disposition: form-data; name="street"',
+            b'',
+            b'\xe6\xb1\x89\xe8\xaf\xad/\xe6\xbc\xa2\xe8\xaa\x9e',
+            b'---123--',
+            b'',
+        ])
+        request = self._createRequest(extra, body)
+        publish(request)
+        self.assertTrue(isinstance(request.form[u"street"], unicode))
+        self.assertEqual(u"汉语/漢語", request.form['street'])
+
+    def testFormURLEncodedUTF8(self):
+        extra = {
+            'REQUEST_METHOD': 'POST',
+            'CONTENT_TYPE': 'application/x-www-form-urlencoded',
+        }
+        body = b'a=5&b:int=6&street=\xe6\xb1\x89\xe8\xaf\xad/\xe6\xbc\xa2\xe8\xaa\x9e'
+        request = self._createRequest(extra, body)
+        publish(request)
+        self.assertTrue(isinstance(request.form[u"street"], unicode))
+        self.assertEqual(u"汉语/漢語", request.form['street'])
+
+    def testFormQueryStringUTF8(self):
+        extra = {
+            'QUERY_STRING': 'a=5&b:int=6&street=\xe6\xb1\x89\xe8\xaf\xad/\xe6\xbc\xa2\xe8\xaa\x9e'
+        }
+        request = self._createRequest(extra)
+        publish(request)
+        self.assertTrue(isinstance(request.form[u"street"], unicode))
+        self.assertEqual(u"汉语/漢語", request.form['street'])
+
+    def testFormURLEncodedLatin1(self):
+        extra = {
+            'REQUEST_METHOD': 'POST',
+            'CONTENT_TYPE': 'application/x-www-form-urlencoded',
+            'HTTP_ACCEPT_CHARSET': 'ISO-8859-1',
+        }
+        body = b'a=5&b:int=6&street=K\xf6hlerstra\xdfe'
+        request = self._createRequest(extra, body)
+        publish(request)
+        self.assertTrue(isinstance(request.form[u"street"], unicode))
+        self.assertEqual(u"K\xf6hlerstra\xdfe", request.form['street'])
+
+    def testFormURLEncodedLatin7(self):
+        extra = {
+            'REQUEST_METHOD': 'POST',
+            'CONTENT_TYPE': 'application/x-www-form-urlencoded',
+            'HTTP_ACCEPT_CHARSET': 'ISO-8859-13',
+        }
+        body = u'a=5&b:int=6&street=Ąžuolyno'.encode('iso-8859-13')
+        request = self._createRequest(extra, body)
+        publish(request)
+        self.assertTrue(isinstance(request.form[u"street"], unicode))
+        self.assertEqual(u"Ąžuolyno", request.form['street'])
 
     def testFormNoEncodingUsesUTF8(self):
-        encoded = 'K\xc3\x83\xc2\xb6hlerstra\xc3\x83\xc2\x9fe'
+        encoded = 'K\xc3\xb6hlerstra\xc3\x9fe'
         extra = {
             # if nothing else is specified, form data should be
             # interpreted as UTF-8, as this stub query string is
@@ -271,206 +385,217 @@ class BrowserTests(HTTPTests):
         # many mainstream browsers do not send HTTP_ACCEPT_CHARSET
         del request._environ['HTTP_ACCEPT_CHARSET']
         publish(request)
-        self.assert_(isinstance(request.form[u'street'], unicode))
-        self.assertEqual(unicode(encoded, 'utf-8'), request.form['street'])
+        self.assertTrue(isinstance(request.form[u"street"], unicode))
+        self.assertEqual(u"K\xf6hlerstra\xdfe", request.form['street'])
+
+    def testFormAcceptsStarButNotUTF8(self):
+        extra = {
+            'QUERY_STRING': 'a=5&b:int=6&latin_1=\xf6', # latin-1
+            'HTTP_ACCEPT_CHARSET': 'utf-8;q=0.7, *;q=0.7',
+            }
+        request = self._createRequest(extra)
+        # don't error when * is in ACCEPT_CHARSET and data is not UTF-8
+        publish(request)
 
     def testFormListTypes(self):
-        extra = {'QUERY_STRING':'a:list=5&a:list=6&b=1'}
+        extra = {'QUERY_STRING': 'a:list=5&a:list=6&b=1'}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'a':[u'5',u'6'], u'b':u'1'})
+        self.assertEqual(request.form, {u"a": [u"5",u"6"], u"b": u"1"})
 
     def testQueryStringIgnoredForPOST(self):
         request = self._createRequest(
             {"REQUEST_METHOD": "POST",
-             'PATH_INFO': '/folder/item3'}, body='c=5&d:int=6')
+             'PATH_INFO': '/folder/item3'}, body=b'c=5&d:int=6')
         publish(request)
-        self.assertEqual(request.form, {u'c': u'5', u'd': 6})
+        self.assertEqual(request.form, {u"c": u"5", u"d": 6})
         self.assertEqual(request.get('QUERY_STRING'), 'a=5&b:int=6')
 
     def testFormTupleTypes(self):
-        extra = {'QUERY_STRING':'a:tuple=5&a:tuple=6&b=1'}
+        extra = {'QUERY_STRING': 'a:tuple=5&a:tuple=6&b=1'}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'a':(u'5',u'6'), u'b':u'1'})
+        self.assertEqual(request.form, {u"a": (u"5", u"6"), u"b": u"1"})
 
     def testFormTupleRecordTypes(self):
-        extra = {'QUERY_STRING':'a.x:tuple:record=5&a.x:tuple:record=6&b=1'}
+        extra = {'QUERY_STRING': 'a.x:tuple:record=5&a.x:tuple:record=6&b=1'}
         request = self._createRequest(extra)
         publish(request)
-        keys = request.form.keys()
-        keys.sort()
-        self.assertEqual(keys, [u'a',u'b'])
-        self.assertEqual(request.form[u'b'], u'1')
-        self.assertEqual(request.form[u'a'].keys(), [u'x'])
-        self.assertEqual(request.form[u'a'][u'x'], (u'5',u'6'))
-        self.assertEqual(request.form[u'a'].x, (u'5',u'6'))
-        self.assertEqual(str(request.form[u'a']), "{x: (u'5', u'6')}")
-        self.assertEqual(repr(request.form[u'a']), "{x: (u'5', u'6')}")
+        keys = sorted(request.form.keys())
+        self.assertEqual(keys, [u"a", u"b"])
+        self.assertEqual(request.form[u"b"], u"1")
+        self.assertEqual(list(request.form[u"a"].keys()), [u"x"])
+        self.assertEqual(request.form[u"a"][u"x"], (u"5", u"6"))
+        self.assertEqual(request.form[u"a"].x, (u"5", u"6"))
+        self.assertEqual(str(request.form[u"a"]).replace("u'", "'"),
+                         "{x: ('5', '6')}")
+        self.assertEqual(repr(request.form[u"a"]).replace("u'", "'"),
+                         "{x: ('5', '6')}")
 
     def testFormRecordsTypes(self):
-        extra = {'QUERY_STRING':'a.x:records=5&a.x:records=6&b=1'}
+        extra = {'QUERY_STRING': 'a.x:records=5&a.x:records=6&b=1'}
         request = self._createRequest(extra)
         publish(request)
-        keys = request.form.keys()
-        keys.sort()
-        self.assertEqual(keys, [u'a',u'b'])
-        self.assertEqual(request.form[u'b'], u'1')
-        self.assertEqual(len(request.form[u'a']), 2)
-        self.assertEqual(request.form[u'a'][0][u'x'], u'5')
-        self.assertEqual(request.form[u'a'][0].x, u'5')
-        self.assertEqual(request.form[u'a'][1][u'x'], u'6')
-        self.assertEqual(request.form[u'a'][1].x, u'6')
-        self.assertEqual(str(request.form[u'a']), "[{x: u'5'}, {x: u'6'}]")
-        self.assertEqual(repr(request.form[u'a']), "[{x: u'5'}, {x: u'6'}]")
+        keys = sorted(request.form.keys())
+        self.assertEqual(keys, [u"a", u"b"])
+        self.assertEqual(request.form[u"b"], u"1")
+        self.assertEqual(len(request.form[u"a"]), 2)
+        self.assertEqual(request.form[u"a"][0][u"x"], u"5")
+        self.assertEqual(request.form[u"a"][0].x, u"5")
+        self.assertEqual(request.form[u"a"][1][u"x"], u"6")
+        self.assertEqual(request.form[u"a"][1].x, u"6")
+        self.assertEqual(str(request.form[u"a"]).replace("u'", "'"),
+                         "[{x: '5'}, {x: '6'}]")
+        self.assertEqual(repr(request.form[u"a"]).replace("u'", "'"),
+                         "[{x: '5'}, {x: '6'}]")
 
     def testFormMultipleRecordsTypes(self):
-        extra = {'QUERY_STRING':'a.x:records:int=5&a.y:records:int=51'
+        extra = {'QUERY_STRING': 'a.x:records:int=5&a.y:records:int=51'
             '&a.x:records:int=6&a.y:records:int=61&b=1'}
         request = self._createRequest(extra)
         publish(request)
-        keys = request.form.keys()
-        keys.sort()
-        self.assertEqual(keys, [u'a',u'b'])
-        self.assertEqual(request.form[u'b'], u'1')
-        self.assertEqual(len(request.form[u'a']), 2)
-        self.assertEqual(request.form[u'a'][0][u'x'], 5)
-        self.assertEqual(request.form[u'a'][0].x, 5)
-        self.assertEqual(request.form[u'a'][0][u'y'], 51)
-        self.assertEqual(request.form[u'a'][0].y, 51)
-        self.assertEqual(request.form[u'a'][1][u'x'], 6)
-        self.assertEqual(request.form[u'a'][1].x, 6)
-        self.assertEqual(request.form[u'a'][1][u'y'], 61)
-        self.assertEqual(request.form[u'a'][1].y, 61)
-        self.assertEqual(str(request.form[u'a']),
+        keys = sorted(request.form.keys())
+        self.assertEqual(keys, [u"a", u"b"])
+        self.assertEqual(request.form[u"b"], u"1")
+        self.assertEqual(len(request.form[u"a"]), 2)
+        self.assertEqual(request.form[u"a"][0][u"x"], 5)
+        self.assertEqual(request.form[u"a"][0].x, 5)
+        self.assertEqual(request.form[u"a"][0][u"y"], 51)
+        self.assertEqual(request.form[u"a"][0].y, 51)
+        self.assertEqual(request.form[u"a"][1][u"x"], 6)
+        self.assertEqual(request.form[u"a"][1].x, 6)
+        self.assertEqual(request.form[u"a"][1][u"y"], 61)
+        self.assertEqual(request.form[u"a"][1].y, 61)
+        self.assertEqual(str(request.form[u"a"]),
             "[{x: 5, y: 51}, {x: 6, y: 61}]")
-        self.assertEqual(repr(request.form[u'a']),
+        self.assertEqual(repr(request.form[u"a"]),
             "[{x: 5, y: 51}, {x: 6, y: 61}]")
 
     def testFormListRecordTypes(self):
-        extra = {'QUERY_STRING':'a.x:list:record=5&a.x:list:record=6&b=1'}
+        extra = {'QUERY_STRING': 'a.x:list:record=5&a.x:list:record=6&b=1'}
         request = self._createRequest(extra)
         publish(request)
-        keys = request.form.keys()
-        keys.sort()
-        self.assertEqual(keys, [u'a',u'b'])
-        self.assertEqual(request.form[u'b'], u'1')
-        self.assertEqual(request.form[u'a'].keys(), [u'x'])
-        self.assertEqual(request.form[u'a'][u'x'], [u'5',u'6'])
-        self.assertEqual(request.form[u'a'].x, [u'5',u'6'])
-        self.assertEqual(str(request.form[u'a']), "{x: [u'5', u'6']}")
-        self.assertEqual(repr(request.form[u'a']), "{x: [u'5', u'6']}")
+        keys = sorted(request.form.keys())
+        self.assertEqual(keys, [u"a", u"b"])
+        self.assertEqual(request.form[u"b"], u"1")
+        self.assertEqual(list(request.form[u"a"].keys()), [u"x"])
+        self.assertEqual(request.form[u"a"][u"x"], [u"5", u"6"])
+        self.assertEqual(request.form[u"a"].x, [u"5", u"6"])
+        self.assertEqual(str(request.form[u"a"]).replace("u'", "'"),
+                         "{x: ['5', '6']}")
+        self.assertEqual(repr(request.form[u"a"]).replace("u'", "'"),
+                         "{x: ['5', '6']}")
 
     def testFormListTypes2(self):
-        extra = {'QUERY_STRING':'a=5&a=6&b=1'}
+        extra = {'QUERY_STRING': 'a=5&a=6&b=1'}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'a':[u'5',u'6'], u'b':u'1'})
+        self.assertEqual(request.form, {u"a": [u"5", u"6"], u"b": u"1"})
 
     def testFormIntTypes(self):
-        extra = {'QUERY_STRING':'a:int=5&b:int=-5&c:int=0&d:int=-0'}
+        extra = {'QUERY_STRING': 'a:int=5&b:int=-5&c:int=0&d:int=-0'}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'a': 5, u'b': -5, u'c': 0, u'd': 0})
+        self.assertEqual(request.form, {u"a": 5, u"b": -5, u"c": 0, u"d": 0})
 
-        extra = {'QUERY_STRING':'a:int='}
+        extra = {'QUERY_STRING': 'a:int='}
         request = self._createRequest(extra)
         self.assertRaises(ValueError, publish, request)
 
-        extra = {'QUERY_STRING':'a:int=abc'}
+        extra = {'QUERY_STRING': 'a:int=abc'}
         request = self._createRequest(extra)
         self.assertRaises(ValueError, publish, request)
 
     def testFormFloatTypes(self):
-        extra = {'QUERY_STRING':'a:float=5&b:float=-5.01&c:float=0'}
+        extra = {'QUERY_STRING': 'a:float=5&b:float=-5.01&c:float=0'}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'a': 5.0, u'b': -5.01, u'c': 0.0})
+        self.assertEqual(request.form, {u"a": 5.0, u"b": -5.01, u"c": 0.0})
 
-        extra = {'QUERY_STRING':'a:float='}
+        extra = {'QUERY_STRING': 'a:float='}
         request = self._createRequest(extra)
         self.assertRaises(ValueError, publish, request)
 
-        extra = {'QUERY_STRING':'a:float=abc'}
+        extra = {'QUERY_STRING': 'a:float=abc'}
         request = self._createRequest(extra)
         self.assertRaises(ValueError, publish, request)
 
     def testFormLongTypes(self):
-        extra = {'QUERY_STRING':'a:long=99999999999999&b:long=0L'}
+        extra = {'QUERY_STRING': 'a:long=99999999999999&b:long=0L'}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'a': 99999999999999, u'b': 0})
+        self.assertEqual(request.form, {u"a": 99999999999999, u"b": 0})
 
-        extra = {'QUERY_STRING':'a:long='}
+        extra = {'QUERY_STRING': 'a:long='}
         request = self._createRequest(extra)
         self.assertRaises(ValueError, publish, request)
 
-        extra = {'QUERY_STRING':'a:long=abc'}
+        extra = {'QUERY_STRING': 'a:long=abc'}
         request = self._createRequest(extra)
         self.assertRaises(ValueError, publish, request)
 
     def testFormTokensTypes(self):
-        extra = {'QUERY_STRING':'a:tokens=a%20b%20c%20d&b:tokens='}
+        extra = {'QUERY_STRING': 'a:tokens=a%20b%20c%20d&b:tokens='}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'a': [u'a', u'b', u'c', u'd'],
-                         u'b': []})
+        self.assertEqual(request.form, {u"a": [u"a", u"b", u"c", u"d"],
+                         u"b": []})
 
     def testFormStringTypes(self):
-        extra = {'QUERY_STRING':'a:string=test&b:string='}
+        extra = {'QUERY_STRING': 'a:string=test&b:string='}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'a': u'test', u'b': u''})
+        self.assertEqual(request.form, {u"a": u"test", u"b": u""})
 
     def testFormLinesTypes(self):
-        extra = {'QUERY_STRING':'a:lines=a%0ab%0ac%0ad&b:lines='}
+        extra = {'QUERY_STRING': 'a:lines=a%0ab%0ac%0ad&b:lines='}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'a': [u'a', u'b', u'c', u'd'],
-                         u'b': []})
+        self.assertEqual(request.form, {u"a": [u"a", u"b", u"c", u"d"],
+                         u"b": []})
 
     def testFormTextTypes(self):
-        extra = {'QUERY_STRING':'a:text=a%0a%0db%0d%0ac%0dd%0ae&b:text='}
+        extra = {'QUERY_STRING': 'a:text=a%0a%0db%0d%0ac%0dd%0ae&b:text='}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'a': u'a\nb\nc\nd\ne', u'b': u''})
+        self.assertEqual(request.form, {u"a": u"a\nb\nc\nd\ne", u"b": u""})
 
     def testFormRequiredTypes(self):
-        extra = {'QUERY_STRING':'a:required=%20'}
+        extra = {'QUERY_STRING': 'a:required=%20'}
         request = self._createRequest(extra)
         self.assertRaises(ValueError, publish, request)
 
     def testFormBooleanTypes(self):
-        extra = {'QUERY_STRING':'a:boolean=&b:boolean=1&c:boolean=%20'}
+        extra = {'QUERY_STRING': 'a:boolean=&b:boolean=1&c:boolean=%20'}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'a': False, u'b': True, u'c': True})
+        self.assertEqual(request.form, {u"a": False, u"b": True, u"c": True})
 
     def testFormDefaults(self):
-        extra = {'QUERY_STRING':'a:default=10&a=6&b=1'}
+        extra = {'QUERY_STRING': 'a:default=10&a=6&b=1'}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'a':u'6', u'b':u'1'})
+        self.assertEqual(request.form, {u"a": u"6", u"b": u"1"})
 
     def testFormDefaults2(self):
-        extra = {'QUERY_STRING':'a:default=10&b=1'}
+        extra = {'QUERY_STRING': 'a:default=10&b=1'}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'a':u'10', u'b':u'1'})
+        self.assertEqual(request.form, {u"a": u"10", u"b": u"1"})
 
     def testFormFieldName(self):
-        extra = {'QUERY_STRING':'c+%2B%2F%3D%26c%3Aint=6',
+        extra = {'QUERY_STRING': 'c+%2B%2F%3D%26c%3Aint=6',
                  'PATH_INFO': '/folder/item3/'}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'c +/=&c': 6})
+        self.assertEqual(request.form, {u"c +/=&c": 6})
 
     def testFormFieldValue(self):
-        extra = {'QUERY_STRING':'a=b+%2B%2F%3D%26b%3Aint',
+        extra = {'QUERY_STRING': 'a=b+%2B%2F%3D%26b%3Aint',
                  'PATH_INFO': '/folder/item3/'}
         request = self._createRequest(extra)
         publish(request)
-        self.assertEqual(request.form, {u'a':u'b +/=&b:int'})
+        self.assertEqual(request.form, {u"a": u"b +/=&b:int"})
 
     def testInterface(self):
         request = self._createRequest()
@@ -492,12 +617,12 @@ class BrowserTests(HTTPTests):
 
     def testIssue559(self):
         extra = {'QUERY_STRING': 'HTTP_REFERER=peter',
-                 'HTTP_REFERER':'http://localhost/',
+                 'HTTP_REFERER': 'http://localhost/',
                  'PATH_INFO': '/folder/item3/'}
         request = self._createRequest(extra)
         publish(request)
         self.assertEqual(request.headers.get('HTTP_REFERER'), 'http://localhost/')
-        self.assertEqual(request.form, {u'HTTP_REFERER': u'peter'})
+        self.assertEqual(request.form, {u"HTTP_REFERER": u"peter"})
 
 
     def test_post_body_not_consumed_unnecessarily(self):
@@ -505,9 +630,9 @@ class BrowserTests(HTTPTests):
             dict(REQUEST_METHOD='POST',
                  CONTENT_TYPE='application/x-foo',
                  ),
-            'test body')
+            b'test body')
         request.processInputs()
-        self.assertEqual(request.bodyStream.read(), 'test body')
+        self.assertEqual(request.bodyStream.read(), b'test body')
 
     def test_post_body_not_necessarily(self):
         request = self._createRequest(
@@ -515,9 +640,9 @@ class BrowserTests(HTTPTests):
                  CONTENT_TYPE='application/x-www-form-urlencoded',
                  QUERY_STRING='',
                  ),
-            'x=1&y=2')
+            b'x=1&y=2')
         request.processInputs()
-        self.assertEqual(request.bodyStream.read(), '')
+        self.assertEqual(request.bodyStream.read(), b'')
         self.assertEqual(dict(request.form), dict(x='1', y='2'))
 
         request = self._createRequest(
@@ -526,13 +651,13 @@ class BrowserTests(HTTPTests):
                                '; charset=UTF-8'),
                  QUERY_STRING='',
                  ),
-            'x=1&y=2')
+            b'x=1&y=2')
         request.processInputs()
-        self.assertEqual(request.bodyStream.read(), '')
+        self.assertEqual(request.bodyStream.read(), b'')
         self.assertEqual(dict(request.form), dict(x='1', y='2'))
 
+@implementer(IBrowserPublication)
 class TestBrowserPublication(TestPublication):
-    implements(IBrowserPublication)
 
     def getDefaultTraversal(self, request, ob):
         return ob, ()
@@ -545,11 +670,11 @@ class APITests(BaseTestIPublicationRequest,
     def _Test__new(self, environ=None, **kw):
         if environ is None:
             environ = kw
-        return BrowserRequest(StringIO(''), environ)
+        return BrowserRequest(BytesIO(b''), environ)
 
     def test_IApplicationRequest_bodyStream(self):
-        request = BrowserRequest(StringIO('spam'), {})
-        self.assertEqual(request.bodyStream.read(), 'spam')
+        request = BrowserRequest(BytesIO(b'spam'), {})
+        self.assertEqual(request.bodyStream.read(), b'spam')
 
     # Needed by BaseTestIEnumerableMapping tests:
     def _IEnumerableMapping__stateDict(self):
