@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001-2002 Zope Foundation and Contributors.
+# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,8 +12,6 @@
 #
 ##############################################################################
 """Test Publisher
-
-$Id: test_publisher.py 105429 2009-11-02 07:47:51Z ctheune $
 """
 import unittest
 
@@ -22,13 +20,16 @@ from zope.publisher.publish import publish, DoNotReRaiseException
 from zope.publisher.base import TestRequest
 from zope.publisher.base import DefaultPublication
 from zope.publisher.interfaces import Unauthorized, NotFound, DebugError
-from zope.publisher.interfaces import IPublication, IReRaiseException
+from zope.publisher.interfaces import IPublication, IReRaiseException, \
+                                      Retry
 
 from zope.interface.verify import verifyClass
 from zope.interface import implementedBy
 
-from StringIO import StringIO
+from io import BytesIO
 
+class ErrorToRetry(Exception):
+    """A sample exception that should be retried."""
 
 class PublisherTests(unittest.TestCase):
     def setUp(self):
@@ -47,18 +48,24 @@ class PublisherTests(unittest.TestCase):
             def __call__(self):
                 return "Yo! No docstring!"
 
+        class RetryItem:
+            """An item that the publication will attempt to retry."""
+            def __call__(self):
+                raise ErrorToRetry()
+
         self.app = AppRoot()
         self.app.folder = Folder()
         self.app.folder.item = Item()
 
         self.app._item = Item()
         self.app.noDocString = NoDocstringItem()
+        self.app.retryItem = RetryItem()
 
     def _createRequest(self, path, **kw):
         publication = DefaultPublication(self.app)
         path = path.split('/')
         path.reverse()
-        request = TestRequest(StringIO(''), **kw)
+        request = TestRequest(BytesIO(b''), **kw)
         request.setTraversalStack(path)
         request.setPublication(publication)
         return request
@@ -79,7 +86,7 @@ class PublisherTests(unittest.TestCase):
             provided=IReRaiseException)
 
     def testImplementsIPublication(self):
-        self.failUnless(IPublication.providedBy(
+        self.assertTrue(IPublication.providedBy(
                             DefaultPublication(self.app)))
 
     def testInterfacesVerify(self):
@@ -88,11 +95,11 @@ class PublisherTests(unittest.TestCase):
 
     def testTraversalToItem(self):
         res = self._publisherResults('/folder/item')
-        self.failUnlessEqual(res, 'item')
+        self.assertEqual(res, 'item')
         res = self._publisherResults('/folder/item/')
-        self.failUnlessEqual(res, 'item')
+        self.assertEqual(res, 'item')
         res = self._publisherResults('folder/item')
-        self.failUnlessEqual(res, 'item')
+        self.assertEqual(res, 'item')
 
     def testUnderscoreUnauthorizedException(self):
         self.assertRaises(Unauthorized, self._publisherResults, '/_item')
@@ -127,8 +134,37 @@ class PublisherTests(unittest.TestCase):
         except:
             pass
         self._unregisterExcAdapter(doReRaiseAdapter)
-        self.failUnlessEqual(raised, True)
-            
+        self.assertEqual(raised, True)
+
+    def testRetryErrorIsUnwrapped(self):
+        test = self
+        class RetryPublication(DefaultPublication):
+            def handleException(self, object, request, exc_info,
+                                retry_allowed=True):
+                test.assertTrue(issubclass(exc_info[0], ErrorToRetry))
+                raise Retry(exc_info)
+
+        request = self._createRequest('/retryItem')
+        request.setPublication(RetryPublication(self.app))
+        self.assertFalse(request.supportsRetry())
+        self.assertRaises(ErrorToRetry, publish, request, handle_errors=False)
+
+    def testBareRetryErrorPassedThrough(self):
+        test = self
+        class RetryPublication(DefaultPublication):
+            def handleException(self, object, request, exc_info,
+                                retry_allowed=True):
+                test.assertTrue(issubclass(exc_info[0], ErrorToRetry))
+                raise Retry()
+
+        request = self._createRequest('/retryItem')
+        request.setPublication(RetryPublication(self.app))
+        self.assertFalse(request.supportsRetry())
+        # Retry exception is passed through because it doesn't contain
+        # an original exception.
+        self.assertRaises(Retry, publish, request, handle_errors=False)
+
+
 def test_suite():
     loader = unittest.TestLoader()
     return loader.loadTestsFromTestCase(PublisherTests)
